/* Handle XAudio bitmaps:
 * - xbm to Pixmap conversion.
 * - draw Pixmaps to screen.
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include "action.h"

/* from xaudio.c */
extern Display *display;	/* X struct for server connection */
extern Window	MainWindow;	/* rsc identifier for window context */
extern GC	gc;		/* -"- for graphics */

/* XAudio button bitmaps */
#include "bitmaps/prev.bit"
#include "bitmaps/previ.bit"
#include "bitmaps/play.bit"
#include "bitmaps/playi.bit"
#include "bitmaps/stop.bit"
#include "bitmaps/stopi.bit"
#include "bitmaps/next.bit"
#include "bitmaps/nexti.bit"
#include "bitmaps/up.bit"
#include "bitmaps/upi.bit"
#include "bitmaps/place.bit"
#include "bitmaps/down.bit"
#include "bitmaps/downi.bit"
#include "bitmaps/signed.bit"
#include "bitmaps/unsigned.bit"

/* sizes */
#define TEXT_FONT	"fixed"
#define FONT_HEIGHT	10
#define TEXT_SPACE	2	/* distance from border */

/* horizontal placement for buttons */
#define PREV_X	0
#define STOP_X	prev_width
#define PLAY_X	(STOP_X + stop_width)
#define NEXT_X	(PLAY_X + play_width)

#define PLACE_X	(NEXT_X + next_width)
#define RATE_X	(PLACE_X + place_width / 2 - 2 * TEXT_SPACE)
#define DOWN_X	(PLACE_X + place_width)
#define UP_X	(DOWN_X + down_width)
#define SIGN_X	(UP_X + up_width)

/* vertical placement */
#define TEXT_Y	(FONT_HEIGHT + TEXT_SPACE)		/* sample name */
#define STRIP_Y	(FONT_HEIGHT + 2 * TEXT_SPACE)		/* button strip */
#define RATE_Y	(STRIP_Y + FONT_HEIGHT + 2 * TEXT_SPACE)


int WinWidth =  prev_width + play_width + stop_width + next_width +
		place_width + up_width + down_width + signed_width;
int WinHeight = FONT_HEIGHT + 2 * TEXT_SPACE + prev_height;


struct Rect { int x; int y; int w; int h; char opcode; };
/* App's X bitmaps */
static Pixmap bmap[BITMAPS][2];
static struct Rect Place[BITMAPS];
#define SETMAP(i,xx,yy,ww,hh,op) {Place[i].x = xx; Place[i].y = yy; \
		Place[i].w = ww; Place[i].h = hh; Place[i].opcode = op;}

/* convert XBMs to X pixmaps */
void AssignBitmaps()
{
  bmap[PREV][NORMAL] = XCreateBitmapFromData(display, MainWindow, prev_bits, prev_width, prev_height);
  bmap[STOP][NORMAL] = XCreateBitmapFromData(display, MainWindow, stop_bits, stop_width, stop_height);
  bmap[PLAY][NORMAL] = XCreateBitmapFromData(display, MainWindow, play_bits, play_width, play_height);
  bmap[NEXT][NORMAL] = XCreateBitmapFromData(display, MainWindow, next_bits, next_width, next_height);
  bmap[DOWN][NORMAL] = XCreateBitmapFromData(display, MainWindow, down_bits, down_width, down_height);
  bmap[UPPE][NORMAL] = XCreateBitmapFromData(display, MainWindow, up_bits, up_width, up_height);
  bmap[SIGN][NORMAL] = XCreateBitmapFromData(display, MainWindow, signed_bits, signed_width, signed_height);
  bmap[PLACE][NORMAL]= XCreateBitmapFromData(display, MainWindow, place_bits, place_width, place_height);

  bmap[PREV][REVERSE] = XCreateBitmapFromData(display, MainWindow, previ_bits, previ_width, previ_height);
  bmap[STOP][REVERSE] = XCreateBitmapFromData(display, MainWindow, stopi_bits, stopi_width, stopi_height);
  bmap[PLAY][REVERSE] = XCreateBitmapFromData(display, MainWindow, playi_bits, playi_width, playi_height);
  bmap[NEXT][REVERSE] = XCreateBitmapFromData(display, MainWindow, nexti_bits, nexti_width, nexti_height);
  bmap[DOWN][REVERSE] = XCreateBitmapFromData(display, MainWindow, downi_bits, downi_width, downi_height);
  bmap[UPPE][REVERSE] = XCreateBitmapFromData(display, MainWindow, upi_bits, upi_width, upi_height);
  bmap[SIGN][REVERSE] = XCreateBitmapFromData(display, MainWindow, unsigned_bits, unsigned_width, unsigned_height);
  bmap[PLACE][REVERSE]= XCreateBitmapFromData(display, MainWindow, place_bits, place_width, place_height);

  /* set bitmap place and size to the struct */
  SETMAP(PREV, PREV_X, STRIP_Y, prev_width, prev_height, OpPREV);
  SETMAP(STOP, STOP_X, STRIP_Y, stop_width, stop_height, OpSTOP);
  SETMAP(PLAY, PLAY_X, STRIP_Y, play_width, play_height, OpPLAY);
  SETMAP(NEXT, NEXT_X, STRIP_Y, next_width, next_height, OpNEXT);
  SETMAP(DOWN, DOWN_X, STRIP_Y, down_width, down_height, OpDOWN);
  SETMAP(UPPE, UP_X, STRIP_Y, up_width, up_height, OpUPPE);
  SETMAP(SIGN, SIGN_X, STRIP_Y, signed_width, signed_height, OpSIGN);
  SETMAP(PLACE, PLACE_X, STRIP_Y, place_width, place_height, 0);
}

/* check if button pressed and return it's opcode */
char ProcessXbutton(XButtonEvent *button)
{
  int i;
  for(i = 0; i < BITMAPS; i++)
    if(button->x > Place[i].x && button->y > Place[i].y &&
       button->x < (Place[i].x + Place[i].w) &&
       button->y < (Place[i].y + Place[i].h))
      return Place[i].opcode;

  return 0;
}

/* Show sample name and lenght in seconds */
void ShowSample(char *name, int secs)
{
  static char output[32];
  sprintf(output, " %s, %dsecs ", name, secs);
  XDrawImageString(display, MainWindow, gc, 0, TEXT_Y, output, strlen(output));
}

/* show sample rate in the window */
void ShowRate(int rate)
{
  static char strRate[4];
  sprintf(strRate, "%d", rate);
  XCopyPlane(display, bmap[PLACE][NORMAL], MainWindow, gc,
    0, 0, Place[PLACE].w, Place[PLACE].h, Place[PLACE].x, Place[PLACE].y, 1UL);
  XDrawImageString(display, MainWindow, gc, RATE_X, RATE_Y, strRate, strlen(strRate));
}

/* Reverse and normalize button i */
void ReverseButton(int i)
{
  XCopyPlane(display, bmap[i][REVERSE], MainWindow, gc,
    0, 0, Place[i].w, Place[i].h, Place[i].x, Place[i].y, 1UL);
}
void NormalButton(int i)
{
  XCopyPlane(display, bmap[i][NORMAL], MainWindow, gc,
    0, 0, Place[i].w, Place[i].h, Place[i].x, Place[i].y, 1UL);
}
