/* X user interface for the XAudio */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include "xaudio.icon"		/* window icon */
#include "action.h"		/* FILENAMES */

/* from action.c */
extern char *FileNames[];
extern int LoadFile(char *filename);
extern int DoAction(char action);
extern void Redraw(void);
extern void DoClean();

/* from bitmaps.c */
extern int WinWidth;
extern int WinHeight;
extern void AssignBitmaps(void);
extern char ProcessXbutton(XButtonEvent *button);

/* standard X app structs */
Display *display;	/* X struct for server connection */
Window	MainWindow;	/* rsc identifier for window context */
GC	gc;		/* -"- for graphics */

/* Initialize X things */
int Xinit(int argc, char *argv[])
{
  static XSizeHints	hint;		/* informs other apps about this one */
  static Pixmap		icon;		/* window icon struct */
  static char xaudio[] = "XAudio v1.0";

  unsigned long foreground, background;
  int screen;

  /* Connect to X server */
  if(!(display = XOpenDisplay("")))	/* connect to $DISPLAY */
  {
    fputs("Unable to open $DISPLAY\n", stderr);
    return 0;
  }
  screen = DefaultScreen(display);

  /* Device independent colors */
  background = WhitePixel(display, screen);
  foreground = BlackPixel(display, screen);

  /* Default window size (flags indicates non-changable attributes) */
  hint.x = hint.y = 10;
  hint.width = WinWidth;
  hint.height = WinHeight;
  hint.flags = PSize;

  /* create win sctruct to root window with a border width of 5 */
  MainWindow = XCreateSimpleWindow(display,
             DefaultRootWindow(display),
	     hint.x, hint.y, hint.width, hint.height,
	     5, foreground, background);

  /* announce the window (struct, name, size, icon...) to the other apps */
  icon = XCreateBitmapFromData(display, MainWindow, xaudio_bits, xaudio_width, xaudio_height);
  XSetStandardProperties(display, MainWindow, xaudio, xaudio, icon, argv, argc, &hint);

  /* convert app XBMs to X pixmaps */
  AssignBitmaps();

  /* Create a graphic context and set it's attributes */
  gc = XCreateGC(display, MainWindow, 0, 0);
  XSetBackground(display, gc, background);
  XSetForeground(display, gc, foreground);

  /* set the event input mask */
  XSelectInput(display, MainWindow, ButtonPressMask | KeyPressMask | ExposureMask);

  /* raise (draw) the window (results in expose event) */
  XMapRaised(display, MainWindow);

  return 1;
}


/* close window and disconnect from the X server */
void Xexit(void)
{
  XFreeGC(display, gc);
  XDestroyWindow(display, MainWindow);
  XCloseDisplay(display);
}


/* returns the char or zero if more than one */
char ProcessXkey(XKeyEvent *kev)
{
#define BUF_SIZE 16
  static char text[BUF_SIZE];
  static KeySym mykey;		/* for precise id */

  /* translates XEvent string into a char string, returns lenght */
  if(XLookupString(kev, text, BUF_SIZE, &mykey, 0) == 1)
    return(text[0]);
  return(0);
#undef BUF_SIZE
}


/* get and process user / X events */
int XEventLoop(void)
{
  static XEvent event;		/* struct for input events */
  char key;

  /* The main loop */
  for(;;)
  {
    /* process the masked events */
    XNextEvent(display, &event);
    switch(event.type)
    {
      /* Process character input */
      case KeyPress:
        if(!DoAction(key = ProcessXkey(&event.xkey)))
	  return 0;
	else if(key == 'q' || key == 'Q' || key == '\03')
	  return 0;
        break;

      /* Process mouse clicks */
      case ButtonPress:
        if(!DoAction(key=ProcessXbutton(&event.xbutton)))
	  return 0;
	break;

      /* Redraw window */
      case Expose:
        if(event.xexpose.count == 0)
          Redraw();
	break;

      /* Change keyboard mapping */
      case MappingNotify:
        XRefreshKeyboardMapping(&event.xmapping);
	break;
    }
  }
}

/* check command line */
int CheckCli(int argc, char *argv[])
{
  int i = 0, files = 0;
  FILE *fp;

  while(++i < argc)
  {
    if(argv[i][0] == '-')
      i++;
    else
      if((fp = fopen(argv[i], "rb")))
      {
        if(files + 1 < FILENAMES)
          FileNames[files++] = argv[i];
	fclose(fp);
      }
  }
  FileNames[files] = NULL;

  if(files)
    return 1;
  return 0;		/* no files to play */
}

/* start XAudio */
int main(int argc, char *argv[])
{
  if(CheckCli(argc, argv))
  {
    if(Xinit(argc, argv))
    {
      if(LoadFile(FileNames[0]))
      {
        XEventLoop();
        DoClean();		/* free mallocated */
      }
      Xexit();
    }
  }
  else
    fprintf(stderr, "Usage: %s [sample1 [sample2...]]", argv[0]);
  return 0;
}
